@@ -25,7 +25,6 @@ Property Juggling
 .. automethod:: synapseclient.utils.get_entity_type
 .. automethod:: synapseclient.utils.is_url
 .. automethod:: synapseclient.utils.as_url
-.. automethod:: synapseclient.utils.is_synapse_entity
 .. automethod:: synapseclient.utils.is_synapse_id
 .. automethod:: synapseclient.utils.to_unix_epoch_time
 .. automethod:: synapseclient.utils.from_unix_epoch_time
@@ -62,6 +61,7 @@ import platform
 from datetime import datetime as Datetime
 from datetime import date as Date
 from numbers import Number
+from synapseclient.exceptions import *
 
 UNIX_EPOCH = Datetime(1970, 1, 1, 0, 0)
 ISO_FORMAT = "%Y-%m-%dT%H:%M:%S.000Z"
@@ -127,8 +127,8 @@ def guess_object_type(obj):
     if isinstance(obj, basestring):
         if obj.startswith('syn'):
             return 'entity'
-    elif 'entityType' in obj:
-        return 'entity'
+    # elif 'entityType' in obj:
+    #     return 'entity'
     elif 'contentSource' in obj:
         return 'evaluation'
     else:
@@ -168,7 +168,7 @@ def id_of(obj):
         return str(obj)
     result = _get_from_members_items_or_properties(obj, 'id')
     if result is None:
-        raise Exception('Invalid parameters: couldn\'t find id of ' + str(obj))
+        raise SynapseMalformedEntityError('Invalid parameters: couldn\'t find id of ' + str(obj))
     return result
 
 
@@ -190,7 +190,7 @@ def get_properties(entity):
 def get_entity_type(entity):
     """Returns the Entity's type."""
     
-    return _get_from_members_items_or_properties(entity, 'entityType')
+    return _get_from_members_items_or_properties(entity, 'concreteType')
 
 
 def is_url(s):
@@ -242,13 +242,6 @@ def file_url_to_path(url, verify_exists=False):
                 'cacheDir': os.path.dirname(path) }
     return {}
 
-def is_synapse_entity(entity):
-    """TODO_Sphinx"""
-    
-    if isinstance(entity, collections.Mapping):
-        return 'entityType' in entity
-    return False
-
 
 def is_synapse_id(obj):
     """Returns None iff the input is a Synapse ID."""
