@@ -34,10 +34,10 @@ Wiki Class
 
 """
 
-from exceptions import ValueError
 import sys
 import json
 
+from synapseclient.exceptions import *
 from synapseclient.dict_object import DictObject
 from synapseclient.utils import guess_object_type, id_of
 
